/* AntSnes
 *
 * Copyright (C) 2009 Summeli <summeli@summeli.fi>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "AntAudio.h"

/*******************************************
 *
 * CAntAudio
 *
 *******************************************/

CAntAudio::CAntAudio(TInt aRate, TBool aStereo, TInt aPcmFrames,  TInt aBufferedFrames)
: iRate(aRate), iStereo(aStereo), iBufferedFrames(aBufferedFrames), iPcmFrames(aPcmFrames)
{
}


CAntAudio* CAntAudio::NewL(TInt aRate, TBool aStereo, TInt aPcmFrames, TInt aBufferedFrames)
{
	CAntAudio* self = new(ELeave) CAntAudio(aRate, aStereo, aPcmFrames, aBufferedFrames);
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);		// self
	return self;
}


CAntAudio::~CAntAudio()
{
	if(iMdaAudioOutputStream) 
		{
		iMdaAudioOutputStream->Stop();
		iWait->Start();
		delete iMdaAudioOutputStream;
		}

	for (TInt i=0 ; i<KSoundBuffers+1 ; i++)
		delete iSoundBuffers[i];

	delete iWait;
	iEmptyBufQ.Reset();
	iEmptyBufQ.Close();
#ifdef OUTPUT_TO_FILE
	iFile.Close();
	iFs.Close();
#endif	
}


void CAntAudio::ConstructL()
{

	switch(iRate) {
		case 11025: iMdaAudioDataSettings.iSampleRate = TMdaAudioDataSettings::ESampleRate11025Hz; break;
		case 16000: iMdaAudioDataSettings.iSampleRate = TMdaAudioDataSettings::ESampleRate16000Hz; break;
		case 22050: iMdaAudioDataSettings.iSampleRate = TMdaAudioDataSettings::ESampleRate22050Hz; break;
		default:    iMdaAudioDataSettings.iSampleRate = TMdaAudioDataSettings::ESampleRate8000Hz;  break;
	}

	iMdaAudioDataSettings.iChannels   = (iStereo) ? TMdaAudioDataSettings::EChannelsStereo : TMdaAudioDataSettings::EChannelsMono;
	iMdaAudioDataSettings.iCaps       = TMdaAudioDataSettings::ESampleRateFixed | iMdaAudioDataSettings.iSampleRate;
	iMdaAudioDataSettings.iFlags      = TMdaAudioDataSettings::ENoNetworkRouting;
	
	//TInt	bytesPerFrame = iStereo ? iPcmFrames << 2 : iPcmFrames << 1;
    TInt bytesPerFrame = iPcmFrames;
	for (TInt i=0 ; i<KSoundBuffers ; i++)
	{
		//TODO: calculate correct buffer size!
		iSoundBuffers[i] = HBufC8::NewL(bytesPerFrame * iBufferedFrames);
		iSoundBuffers[i]->Des().FillZ  (bytesPerFrame * iBufferedFrames );
		TInt err = iEmptyBufQ.Append(i);
		User::LeaveIfError(err);
	}
	// because feeding 2 buffers after an underflow is a little too much, but feeding 1 may be not enough,
	// prepare this ~50ms empty buffer to additionaly feed after every underflow.
	// Another strange thing here: if we try to make and odd-length sound buffer here,
	// system then outputs horrible noise! (this happened on 22050 mono and when there
	// were no parenthesis around iBufferedFrames / 4.
	
	iSoundBuffers[KSoundBuffers] = HBufC8::NewL(bytesPerFrame * (iBufferedFrames / 4));
	iSoundBuffers[KSoundBuffers]->Des().FillZ  (bytesPerFrame * (iBufferedFrames / 4));

	iFrameCount = 0;

	iMdaAudioOutputStream = CMdaAudioOutputStream::NewL(*this);

	iWait = new (ELeave) CActiveSchedulerWait;

#ifdef OUTPUT_TO_FILE
	iFs.Connect();
	// try to open
	TInt  err = iFile.Open(iFs, _L("C:\\data\\AntSnes\\antaudio.data"), EFileWrite|EFileShareAny);
	if(err) 
		{
		err = iFile.Create(iFs, _L("C:\\data\\AntSnes\\antaudio.data"), EFileWrite|EFileShareAny);
		}	
	if( err )
		{
		err = iFile.Create(iFs, _L("C:\\data\\AntSnes\\audio.data"), EFileWrite|EFileShareAny);
		}
	        
#endif	
}

void CAntAudio::Reset()
	{
	iMdaAudioOutputStream->Open(&iMdaAudioDataSettings);
	iWait->Start();
	
	iMdaAudioOutputStream->SetPriority(EPriorityMuchMore, EMdaPriorityPreferenceNone);
	TInt MaxVol = iMdaAudioOutputStream->MaxVolume();
	TInt vol = (MaxVol * 3) / 10;
	iMdaAudioOutputStream->SetVolume(vol);
	
	iPlaying = EFalse;
	iCurrentPosition = (TUint8*) iSoundBuffers[iEmptyBufQ[0]]->Ptr();
	
	iUnderflowed = ETrue;
	}

// returns a pointer to buffer for next frame,
// to be used when iSoundBuffers are used directly
TUint8 *CAntAudio::NextFrameL()
{
	if( iEmptyBufQ.Count() == 1) //trick to prevent overflows
		{
		return NULL;
		}
	return iCurrentPosition;

}


void CAntAudio::FrameMixed()
	{
	TInt nextFrame = 0;
	TBool bufferComplete = EFalse;
	if( ++iFrameCount == iBufferedFrames )
		{
		nextFrame = iEmptyBufQ[0];
		iEmptyBufQ.Remove(0);
		iEmptyBufQ.Compress();
		TInt nextpos = iEmptyBufQ[0];

		//calculate new current position 
		iCurrentPosition = (TUint8*) iSoundBuffers[nextpos]->Ptr();
		iFrameCount = 0;
		bufferComplete = ETrue;
		}

	else
		{
		//move current position forward by one frame 
		iCurrentPosition += iPcmFrames; //<< (iStereo?1:0);
		}
	
	//if next buffer is complete, play the buffer, or put it on the queue
	if( bufferComplete )
		{
		if( iUnderflowed )
			{
			iMdaAudioOutputStream->WriteL(*iSoundBuffers[KSoundBuffers]);
			iUnderflowed = EFalse;
			}
		iMdaAudioOutputStream->WriteL(*iSoundBuffers[nextFrame]);
		iPlaying = ETrue;
		}
	}

TInt CAntAudio::FreeBufferCount()
	{
	return iEmptyBufQ.Count();
	}

void CAntAudio::MaoscOpenComplete(TInt aError)
{
	iPlaying = EFalse;

	iWait->AsyncStop();
}

void CAntAudio::MaoscBufferCopied(TInt aError, const TDesC8& aBuffer)
{

	if( aBuffer.Ptr() == iSoundBuffers[0]->Ptr())
		iEmptyBufQ.Append(0);
	else if(  aBuffer.Ptr() == iSoundBuffers[1]->Ptr() )
		iEmptyBufQ.Append(1);
	else if(  aBuffer.Ptr() == iSoundBuffers[2]->Ptr() )
		iEmptyBufQ.Append(2);
	else if(  aBuffer.Ptr() == iSoundBuffers[3]->Ptr() )
		iEmptyBufQ.Append(3);
	
	if( iEmptyBufQ.Count() == 4 )//all buffers are empty
		iUnderflowed = ETrue;

}
void CAntAudio::MaoscPlayComplete(TInt aError)
{
	iPlaying = EFalse;
	iUnderflowed = ETrue;
}
